import datetime

import pandas as pd

from .data import data_path, resource, dependency

__all__ = ["load"]

start = datetime.datetime(2019, 1, 1)
day = datetime.timedelta(days=1)
hour = datetime.timedelta(hours=1)


def load():
    _fetch()

    dfs = []

    for i in range(365):
        date = start + i * day

        # Load CSV.
        date_str = date.strftime("%Y%m%d")
        df = pd.read_csv(data_path("miso", f"{date_str}_rt_lmp_final.csv"), header=3)
        # Select LMPs.
        df = df[df["Value"] == "LMP"].set_index("Node")

        # Fix column names and transpose.
        df = df[[f"HE {j + 1}" for j in range(24)]]
        df.columns = [date + j * hour for j in range(24)]
        df = df.T

        dfs.append(df)

    return pd.concat(dfs, sort=True)


def _fetch():
    current = start

    for i in range(1, 12 + 1):
        # Get all the files for that month.
        month = datetime.datetime(2019, i, 1)
        date_str_month = month.strftime("%Y%m")
        resource(
            target=data_path("miso", f"{date_str_month}_rt_lmp_final_csv.zip"),
            url=f"https://docs.misoenergy.org/marketreports/"
            f"{date_str_month}_rt_lmp_final_csv.zip",
        )

        # Unzip the downloaded files until we hit the next month.
        while current.month == i:
            date_str_current = current.strftime("%Y%m%d")
            dependency(
                target=data_path("miso", f"{date_str_current}_rt_lmp_final.csv"),
                source=data_path("miso", f"{date_str_month}_rt_lmp_final_csv.zip"),
                commands=[f"unzip -n {date_str_month}_rt_lmp_final_csv.zip"],
            )
            current += day



import numpy as np
import pandas as pd
from sklearn.linear_model import LinearRegression
from sklearn.gaussian_process import GaussianProcessRegressor
from sklearn.gaussian_process.kernels import ConstantKernel, RBF, WhiteKernel

from .data import data_path, resource

__all__ = ["load"]


def load(detrend_method="gp"):
    _fetch()

    df = pd.read_csv(
        data_path("mauna_loa", "co2_mm_mlo.txt"),
        delim_whitespace=True,
        header=72,
    )
    df = df.iloc[:, [2, 3]]
    df.columns = ["year", "ppm"]
    df = df.set_index("year")

    # Remove missing data.
    df[df < 0] = np.nan
    df.dropna(inplace=True)

    # Also compute trend and detrended version.
    if detrend_method == "gp":
        model = GaussianProcessRegressor(
            kernel=ConstantKernel(1.0) * RBF(1.0) + WhiteKernel(0.1),
            random_state=0,
        )
    elif detrend_method == "linear":
        model = LinearRegression()
    else:
        raise ValueError('`detrend_method` must be "gp" or "linear".')
    model.fit(np.array(df.index)[:, None], df["ppm"])
    trend = model.predict(df.index[:, None])
    df["ppm_trend"] = trend
    df["ppm_detrended"] = df["ppm"] - trend

    return df


def _fetch():
    resource(
        target=data_path("mauna_loa", "co2_mm_mlo.txt"),
        url="ftp://aftp.cmdl.noaa.gov/products/trends/co2/co2_mm_mlo.txt",
    )



from functools import wraps

import lab as B
import numpy as np
import pandas as pd
from plum import Dispatcher, Union

__all__ = ["mse", "smse", "rmse", "srmse", "mae", "mll", "smll", "r2"]

_dispatch = Dispatcher()

_Pandas = Union[pd.DataFrame, pd.Series]
_PandasOrScalar = Union[_Pandas, B.Number]


def _auto_convert(f):
    @wraps(f)
    def wrapped_f(*args):
        converted_args = ()
        for i, arg in enumerate(args):
            # Check if the argument is already a Pandas object.
            if isinstance(arg, (pd.Series, pd.DataFrame)):
                converted_args += (arg,)
                continue

            # It is not already a Pandas object. Attempt to convert.
            if B.rank(arg) == 0:
                converted_args += (arg,)
            elif B.rank(arg) == 1:
                converted_args += (pd.Series(arg),)
            elif B.rank(arg) == 2:
                converted_args += (pd.DataFrame(arg),)
            else:
                raise ValueError(
                    f"Argument {i} has rank {B.rank(arg)}, which "
                    f"cannot be automatically converted to a "
                    f"Pandas object."
                )

        return f(*converted_args)

    return wrapped_f


@_auto_convert
@_dispatch
def mse(mean: _PandasOrScalar, data: _Pandas):
    """Mean squared error.

    Args:
        mean (tensor): Mean of prediction.
        data (tensor): Reference data.

    Returns:
        tensor: Mean squared error.
    """
    return ((mean - data) ** 2).mean()


@_auto_convert
@_dispatch
def smse(mean: _PandasOrScalar, data: _Pandas):
    """Standardised mean squared error.

    Args:
        mean (tensor): Mean of prediction.
        data (tensor): Reference data.

    Returns:
        tensor: Standardised mean squared error.
    """
    return mse(mean, data) / mse(data.mean(), data)


@_auto_convert
@_dispatch
def rmse(mean: _PandasOrScalar, data: _Pandas):
    """Root mean squared error.

    Args:
        mean (tensor): Mean of prediction.
        data (tensor): Reference data.

    Returns:
        tensor: Root mean squared error.
    """
    return mse(mean, data) ** 0.5


@_auto_convert
@_dispatch
def srmse(mean: _PandasOrScalar, data: _Pandas):
    """Standardised root mean squared error.

    Args:
        mean (tensor): Mean of prediction.
        data (tensor): Reference data.

    Returns:
        tensor: Standardised root mean squared error.
    """
    return rmse(mean, data) / rmse(data.mean(), data)


@_auto_convert
@_dispatch
def mae(mean: _PandasOrScalar, data: _Pandas):
    """Mean absolute error.

    Args:
        mean (tensor): Mean of prediction.
        data (tensor): Reference data.

    Returns:
        tensor: Mean absolute error.
    """
    return np.abs(mean - data).mean()


@_auto_convert
@_dispatch
def mll(mean: _PandasOrScalar, variance: _PandasOrScalar, data: _Pandas):
    """Mean log loss.

    Args:
        mean (tensor): Mean of prediction.
        variance (tensor): Variance of prediction.
        data (tensor): Reference data.

    Returns:
        tensor: Mean log loss.
    """
    return (
        0.5 * np.log(2 * np.pi * variance) + 0.5 * (mean - data) ** 2 / variance
    ).mean()


@_auto_convert
@_dispatch
def smll(mean: _PandasOrScalar, variance: _PandasOrScalar, data: _Pandas):
    """Standardised mean log loss.

    Args:
        mean (tensor): Mean of prediction.
        variance (tensor): Variance of prediction.
        data (tensor): Reference data.

    Returns:
        tensor: Standardised mean log loss.
    """
    return mll(mean, variance, data) - mll(data.mean(), data.var(ddof=0), data)


@_auto_convert
@_dispatch
def r2(mean: _PandasOrScalar, data: _Pandas):
    """R-squared.

    Args:
        mean (tensor): Mean of prediction.
        data (tensor): Reference data.

    Returns:
        tensor: R-squared.
    """
    return 1 - ((data - mean) ** 2).mean() / data.var(ddof=0)



import datetime
import os

import numpy as np
import pandas as pd

from .data import data_path, split_df, resource

__all__ = ["load"]

cache = data_path("air_temp", "data.parquet")


def _to_float(x):
    try:
        return float(x)
    except ValueError as e:
        if x in {"", "n/a", " "}:
            return np.nan
        elif x.startswith(".-"):
            # Unclear what this means. Just treat it as missing.
            return np.nan
        else:
            raise e


def load(split=None):
    """Load air temperature data.

    Args:
        split (str, optional): Name of split. Must be `None` or `"requeima19"`.

    Returns:
        object: Loaded data. Depends on the value for `split`.
    """
    days = _fetch()

    # Cache the data. Loading the individual files takes a while.
    if not os.path.exists(cache):
        sites = ["Bra", "Cam", "Chi", "Sot"]
        features = [("DEPTH", "height"), ("WSPD", "speed"), ("ATMP", "temp")]

        # Read specified days.
        dfs = []
        for site in sites:
            for day in days:
                month_short = day.strftime("%b")
                try:
                    data = pd.read_csv(
                        data_path(
                            "air_temp",
                            f"{site:s}{day.day:02d}{month_short}{day.year}.csv",
                        )
                    )
                except:
                    # The site might have returned a 404, which is saved as the CSV. Just
                    # ignore these entires.
                    continue

                # Convert times and dates.
                index = []
                for i, row in data.iterrows():
                    day, month, year = map(int, row["Date"].split("/"))
                    hour, minute = map(int, row["Time"].split(":"))
                    dt = datetime.datetime(year, month, day, hour, minute)
                    index.append((site, dt))

                # Construct desired data frame.
                index = pd.MultiIndex.from_tuples(index, names=["site", "date"])
                data = pd.DataFrame(
                    {
                        name_to: list(map(_to_float, data[name_from]))
                        for name_from, name_to in features
                    },
                    index=index,
                )
                dfs.append(data)

        # Concat all and fix order.
        df = pd.concat(dfs)
        # Remove duplications; keep the first.
        df = df[~df.index.duplicated(keep="first")]
        df = df.unstack("site").reindex(
            [
                ("height", "Bra"),
                ("height", "Sot"),
                ("height", "Cam"),
                ("height", "Chi"),
                ("speed", "Bra"),
                ("speed", "Sot"),
                ("speed", "Cam"),
                ("speed", "Chi"),
                ("temp", "Bra"),
                ("temp", "Sot"),
                ("temp", "Cam"),
                ("temp", "Chi"),
            ],
            axis=1,
        )

        # Write cache.
        df.to_parquet(cache)
    else:
        # Cache already exists. Load cache.
        df = pd.read_parquet(cache)

    # Clear invalid sensor readings.
    df[df < -1000] = np.nan

    # Clear an obviously invalid period of readings.
    mask = (
        (datetime.datetime(2014, 1, 1) <= df.index)
        & (df.index < datetime.datetime(2014, 9, 1))
        & (df.temp.Sot == 0)
    )
    df.loc[mask, ("temp", "Sot")] = np.nan

    # TODO: Clear more invalid sensor readings.

    if split is None:
        return df
    elif split == "requeima19":

        def make_range(lower, upper):
            day = datetime.timedelta(days=1)
            date_start = datetime.datetime(year=2013, month=7, day=1)
            return date_start + (lower - 1) * day, date_start + (upper - 1) * day

        # Construct data sets of various sizes.
        ds = []
        for lower, upper in [
            make_range(10, 20),
            make_range(8, 23),
            (
                datetime.datetime(year=2013, month=7, day=1),
                datetime.datetime(year=2013, month=8, day=1),
            ),
        ]:
            data = df[(df.index >= lower) & (df.index < upper)]

            test1, train = split_df(data, make_range(12, 16), [("temp", "Cam")])
            test2, train = split_df(train, make_range(14, 18), [("temp", "Chi")])

            test1_ext, _ = split_df(data, make_range(11.5, 16.5), [("temp", "Cam")])
            test2_ext, _ = split_df(train, make_range(13.5, 18.5), [("temp", "Chi")])

            ds.append((data, train, [test1, test2, test1_ext, test2_ext]))
        return ds
    else:  # pragma: no cover
        raise ValueError(f'Bad split "{split}".')


def _fetch():
    site_urls = [
        ("Cam", "http://www.cambermet.co.uk"),
        ("Chi", "http://www.chimet.co.uk"),
        ("Bra", "http://www.bramblemet.co.uk"),
        ("Sot", "http://www.sotonmet.co.uk"),
    ]
    days = []
    day = datetime.datetime(year=2013, month=1, day=1)
    while day < datetime.datetime(year=2016, month=1, day=1):
        days.append(day)
        day += datetime.timedelta(days=1)
    for day in days:
        month_full = day.strftime("%B")
        month_short = day.strftime("%b")
        for site, url in site_urls:
            resource(
                target=data_path(
                    "air_temp",
                    f"{site}{day.day:02d}{month_short}{day.year}.csv",
                ),
                url=(
                    f"{url}/archive/{day.year}/{month_full}/CSV/"
                    f"{site}{day.day:02d}{month_short}{day.year}.csv"
                ),
            )

    return days



import lab as B
import pandas as pd
import pytest

import wbml.metric

two_arg = ["mse", "smse", "rmse", "srmse", "mae", "r2"]
three_arg = ["mll", "smll"]

two_arg_standardised = [("smse", 1), ("srmse", 1)]
three_arg_standardised = [("smll", 0)]


@pytest.mark.parametrize("name", two_arg)
def test_two_arg(name):
    metric = getattr(wbml.metric, name)

    # Test scalar usage.
    assert isinstance(metric(1, B.randn(10)), B.Number)

    # Test series usage.
    assert isinstance(metric(B.randn(10), B.randn(10)), B.Number)

    # Test matrix usage.
    assert isinstance(metric(B.randn(10, 10), B.randn(10, 10)), pd.Series)

    # Check that higher-order tensors fail.
    with pytest.raises(ValueError):
        metric(B.randn(10, 10, 10), B.randn(10, 10, 10))


@pytest.mark.parametrize("name", three_arg)
def test_three_arg(name):
    metric = getattr(wbml.metric, name)

    # Test scalar usage.
    assert isinstance(metric(1, 1, B.randn(10)), B.Number)

    # Test series usage.
    assert isinstance(metric(B.randn(10), B.rand(10), B.randn(10)), B.Number)

    # Test matrix usage.
    assert isinstance(
        metric(B.randn(10, 10), B.rand(10, 10), B.randn(10, 10)), pd.Series
    )

    # Check that higher-order tensors fail.
    with pytest.raises(ValueError):
        metric(B.randn(10, 10, 10), B.rand(10, 10, 10), B.randn(10, 10, 10))


@pytest.mark.parametrize("name, neutral_value", two_arg_standardised)
def test_two_arg_standardised(name, neutral_value):
    metric = getattr(wbml.metric, name)
    x = B.randn(10)
    mean = x.mean() * B.ones(10)
    assert B.all(metric(mean, x) == neutral_value)


@pytest.mark.parametrize("name, neutral_value", three_arg_standardised)
def test_three_arg_standardised(name, neutral_value):
    metric = getattr(wbml.metric, name)
    x = B.randn(10)
    mean = x.mean() * B.ones(10)
    variance = x.var(ddof=0) * B.ones(10)
    assert B.all(metric(mean, variance, x) == neutral_value)



import subprocess
from functools import wraps

import lab as B
import matplotlib.pyplot as plt
from plum import Dispatcher

from .warning import warn_upmodule

__all__ = ["patch", "tex", "tweak", "style", "pdfcrop"]

_dispatch = Dispatcher()

plt.rcParams["font.size"] = 12
plt.rcParams["mathtext.fontset"] = "cm"  # Use CM for math font.
plt.rcParams["figure.autolayout"] = True  # Use tight layouts.


@_dispatch
def _convert(x):
    return x


@_dispatch
def _convert(x: B.Number):
    return x


@_dispatch
def _convert(x: B.Numeric):
    return B.squeeze(B.to_numpy(x))


@_dispatch
def _convert(xs: tuple):
    return tuple(_convert(x) for x in xs)


@_dispatch
def _convert(xs: list):
    return [_convert(x) for x in xs]


@_dispatch
def _convert(d: dict):
    return {k: _convert(v) for k, v in d.items()}


def patch(f, kind=None):
    """Decorator to patch a function to automatically convert arguments that
    are of a framework type, like PyTorch, to NumPy. Also allows a keyword
    argument `style` to automatically expand the dictionary given by
    :func:`.plot.style` into the keyword arguments.
    """

    @wraps(f)
    def patched_f(*args, **kw_args):
        # Automatically expand style configuration.
        if kind and "style" in kw_args:
            for k, v in style(kw_args["style"], kind).items():
                if k not in kw_args:
                    kw_args[k] = v
            del kw_args["style"]

        return f(*_convert(args), **_convert(kw_args))

    return patched_f


# Patch common plotting functions.
plt.plot = patch(plt.plot, kind="line")
plt.scatter = patch(plt.scatter, kind="scatter")
plt.fill_between = patch(plt.fill_between, kind="fill")
plt.errorbar = patch(plt.errorbar)
plt.imshow = patch(plt.imshow)
plt.xlim = patch(plt.xlim)
plt.ylim = patch(plt.ylim)


def tex():
    """Use TeX for rendering."""
    if subprocess.call("which latex", shell=True) == 0:
        plt.rcParams["text.usetex"] = True  # Use TeX for rendering.
    else:
        warn_upmodule(
            "No LaTeX installation found, so LaTeX will not be used to render "
            "`matplotlib` figures."
        )


def tweak(
    grid=True,
    legend=None,
    legend_loc="upper right",
    spines=True,
    ticks=True,
    ax=None,
):
    """Tweak a plot.

    Args:
        grid (bool, optional): Show grid. Defaults to `True`.
        legend (bool, optional): Show legend. Automatically shows a legend if any labels
            are set.
        legend_loc (str, optional): Position of the legend. Defaults to
            "upper right".
        spines (bool, optional): Hide top and right spine. Defaults to `True`.
        ticks (bool, optional): Hide top and right ticks. Defaults to `True`.
        ax (axis, optional): Axis to tune. Defaults to `plt.gca()`.
    """
    if ax is None:
        ax = plt.gca()

    if grid:
        ax.set_axisbelow(True)  # Show grid lines below other elements.
        ax.grid(which="major", c="#c0c0c0", alpha=0.5, lw=1)

    if legend is None:
        legend = len(ax.get_legend_handles_labels()[0]) > 0

    if legend:
        leg = ax.legend(
            facecolor="#eeeeee",
            framealpha=0.7,
            loc=legend_loc,
            labelspacing=0.25,
        )
        leg.get_frame().set_linewidth(0)

    if spines:
        ax.spines["top"].set_visible(False)
        ax.spines["right"].set_visible(False)
        ax.spines["bottom"].set_lw(1)
        ax.spines["left"].set_lw(1)

    if ticks:
        ax.xaxis.set_ticks_position("bottom")
        ax.xaxis.set_tick_params(width=1)
        ax.yaxis.set_ticks_position("left")
        ax.yaxis.set_tick_params(width=1)


scheme = [
    "#000000",  # Black
    "#F5793A",  # Orange
    "#4BA6FB",  # Modified blue (original was #85C0F9)
    "#A95AA1",  # Pink
    "#689948",  # Green: not optimal for colour-blind people
]
"""list[str]: Color scheme."""

colour_map = {
    "train": scheme[0],
    "test": scheme[1],
    "pred": scheme[2],
    "pred2": scheme[3],
    "pred3": scheme[4],
}
"""dict[str, str]: Name to colour mapping."""

line_style_map = {
    "train": "-",
    "test": "-",
    "pred": "--",
    "pred2": "-.",
    "pred3": (0, (3, 1, 1, 1, 1, 1)),
}
"""dict[str, str]: Name to line style mapping."""

scatter_style_map = {
    "train": "o",
    "test": "x",
    "pred": "s",
    "pred2": "D",
    "pred3": "*",
}
"""dict[str, str]: Name to scatter style mapping."""


def style(name, kind):
    """Generate style setting for functions in :mod:`matplotlib.pyplot`.

    Args:
        name (str): Name of style.
        kind (str): Kind of setting.

    Returns:
        dict: Style settings.
    """
    if kind == "line":
        return {"c": colour_map[name], "ls": line_style_map[name]}
    elif kind == "scatter":
        return {"c": colour_map[name], "marker": scatter_style_map[name], "s": 8}
    elif kind == "fill":
        return {"facecolor": colour_map[name], "alpha": 0.25}
    else:
        return ValueError(f'Unknown kind "{kind}".')


def pdfcrop(path):
    """Run pdfcrop on a PDF.

    Args:
        path (str): Path of PDF.
    """
    if subprocess.call("which pdfcrop", shell=True) == 0:
        subprocess.call(["pdfcrop", path, path])
    else:
        warn_upmodule(
            "External tool `pdfcrop` cannot be found. "
            "Skipping the application of `pdfcrop`."
        )



import lab as B
from stheno import Normal

from wbml.vi import elbo
from .util import approx


def rand_normal(n=3):
    cov = B.randn(n, n)
    cov = B.mm(cov, cov, tr_b=True)
    return Normal(B.randn(n, 1), cov)


def test_elbo():
    lik = rand_normal()
    p = rand_normal()
    q = rand_normal()

    # Check that the two implementations are consistent.
    estimate1 = elbo.invoke(object, Normal, Normal)(lik.logpdf, p, q, num_samples=50000)
    estimate2 = elbo.invoke(object, object, object)(lik.logpdf, p, q, num_samples=50000)
    approx(estimate1, estimate2, rtol=1e-2)



import pandas as pd

from .data import data_path, resource

__all__ = ["load"]


def load():
    """Predator–prey data.

    Source:
        The data is a digitised version of the historical data by D. R. Hundley:
            http://people.whitman.edu/~hundledr/courses/M250F03/LynxHare.txt

    Returns:
        :class:`pd.DataFrame`: Predator–prey data.
    """
    _fetch()

    df = pd.read_csv(
        data_path("pred_prey", "LynxHare.txt"),
        header=None,
        delim_whitespace=True,
        index_col=0,
    )
    df.index.name = "year"
    df.columns = ["hare", "lynx"]

    return df


def _fetch():
    resource(
        target=data_path("pred_prey", "LynxHare.txt"),
        url="http://people.whitman.edu/~hundledr/courses/M250F03/LynxHare.txt",
    )



import xarray as xr
import pandas as pd
import numpy as np

from .data import resource, data_path

__all__ = ["load"]


def load():
    _fetch()

    return {
        "hf": {
            # Temperature two meters above ground from a Regional Climate Model
            # (RMC):
            "temperature": _load_nc(
                data_path("mf_temp", "T2_monthly.nc"),
                ("Time", "time"),
                ("T2", "temperature"),
                ("lat", "latitude"),
                ("lon", "longitude"),
                expand_coords=False,
            )
        },
        "lf": {  # The below is all re-analysis data.
            # Temperature two meters above ground:
            "temperature": _load_nc(
                data_path("mf_temp", "era5_t2m_monthly.nc"),
                ("time", "time"),
                ("t2m", "temperature"),
                ("latitude", "latitude"),
                ("longitude", "longitude"),
                expand_coords=True,
            ),
            # Northbound wind speed:
            "wind_north": _load_nc(
                data_path("mf_temp", "era5_v10_monthly.nc"),
                ("time", "time"),
                ("v10", "wind_north"),
                ("latitude", "latitude"),
                ("longitude", "longitude"),
                expand_coords=True,
            ),
            # Eastbound wind speed:
            "wind_east": _load_nc(
                data_path("mf_temp", "era5_u10_monthly.nc"),
                ("time", "time"),
                ("u10", "wind_east"),
                ("latitude", "latitude"),
                ("longitude", "longitude"),
                expand_coords=True,
            ),
        },
    }


def _load_nc(path, x, y, coord1, coord2, expand_coords=True):
    data = xr.open_dataset(path)

    # Load the variables.
    df = pd.DataFrame(
        _get_reshape(data, y[0]), index=pd.Index(_get_reshape(data, x[0]), name=x[1])
    )

    # Load the coordinates. These can be stored compressedly or expandedly.
    if expand_coords:
        coord1_values = getattr(data, coord1[0]).values
        coord2_values = getattr(data, coord2[0]).values
        # Expand the coordinates through broadcasting.
        coord1_values, coord2_values = np.broadcast_arrays(
            coord1_values[:, None], coord2_values[None, :]
        )
        coords = pd.DataFrame(
            {coord1[1]: coord1_values.reshape(-1), coord2[1]: coord2_values.reshape(-1)}
        )
    else:
        # No expansion is needed.
        coords = pd.DataFrame(
            {
                coord1[1]: _get_reshape(data, coord1[0]),
                coord2[1]: _get_reshape(data, coord2[0]),
            }
        )

    return coords, df


def _get_reshape(data, name):
    values = getattr(data, name).values
    return values.reshape(*values.shape[0:-2], -1)


def _fetch():
    for remote, local in [
        ("HiFid/T2_monthly.nc", "T2_monthly.nc"),
        ("LoFid/era5_t2m_monthly.nc", "era5_t2m_monthly.nc"),
        ("LoFid/era5_v10_monthly.nc", "era5_v10_monthly.nc"),
        ("LoFid/era5_u10_monthly.nc", "era5_u10_monthly.nc"),
    ]:
        resource(
            target=data_path("mf_temp", local),
            url=f"http://gws-access.ceda.ac.uk/public/bas_climate/files/"
            f"scott/wip/multifidelity_modelling/Peru_Simulations/"
            f"monthly_data/{remote}",
        )



import lab as B
import numpy as np
from stheno import Normal

from wbml.util import indented_kv, inv_perm, normal1d_logpdf, BatchVars
from .util import approx


def test_indented_kv():
    assert indented_kv("a", "b") == " a=b"
    assert indented_kv("a", "b\nb2") == " a=b\n   b2"
    assert (
        indented_kv("a", "b\nb2", separator=": ", indent=2, suffix=",\n")
        == "  a: b\n     b2,\n"
    )


def test_inv_perm():
    perm = np.random.permutation(10)
    approx(perm[inv_perm(perm)], B.range(10))


def test_normal1d_logpdf():
    means = B.randn(3, 3)
    covs = B.randn(3, 3) ** 2
    x = B.randn(3, 3)
    logpdfs = normal1d_logpdf(x, covs, means)
    for i in range(3):
        for j in range(3):
            dist = Normal(means[i : i + 1, j : j + 1], covs[i : i + 1, j : j + 1])
            approx(logpdfs[i, j], dist.logpdf(x[i, j]))


def test_batchvars():
    source = B.randn(5, 2 + 3 * 4)
    vs = BatchVars(source=source)
    approx(vs.get(shape=(1, 2)), B.reshape(source[:, :2], 5, 1, 2))
    approx(vs.get(shape=(3, 4)), B.reshape(source[:, 2:], 5, 3, 4))



from datetime import datetime

import pandas as pd

from .data import data_path, resource

__all__ = ["load"]


def load():
    """Crude oil data.

    Source:
        The historical data at the maximum window was downloaded at 9 Oct 2021 from the
        following link:
            https://www.nasdaq.com/market-activity/commodities/cl:nmx/historical

    Returns:
        :class:`pd.DataFrame`: Crude oil data.
    """
    _fetch()

    df = pd.read_csv(data_path("crude_oil", "crude_oil.csv"))
    df.columns = [c.lower() for c in df.columns]
    df.rename(columns={"close/last": "close"})
    df.date = list(map(lambda x: datetime.strptime(x, "%m/%d/%Y"), df.date))
    df.set_index("date", inplace=True)
    df.sort_index(inplace=True)

    return df


def _fetch():
    resource(
        target=data_path("crude_oil", "crude_oil.csv"),
        url="https://www.dropbox.com/s/6ah5l0b64w9n78l/crude_oil.csv?dl=1",
    )



from datetime import datetime

import pandas as pd

from .data import data_path, resource

__all__ = ["load"]


def load():
    """VIX data.

    Source:
        The history starting at 2 Jan 1990 until the time of download is downloaded
        from the following link:
            https://cdn.cboe.com/api/global/us_indices/daily_prices/VIX_History.csv

    Returns:
        :class:`pd.DataFrame`: VIX data.
    """
    _fetch()

    df = pd.read_csv(data_path("vix", "vix.csv"))
    df.columns = [c.lower() for c in df.columns]
    df.rename(columns={"close/last": "close"})
    df.date = list(map(lambda x: datetime.strptime(x, "%m/%d/%Y"), df.date))
    df.set_index("date", inplace=True)
    df.sort_index(inplace=True)

    return df


def _fetch():
    resource(
        target=data_path("vix", "vix.csv"),
        url="https://cdn.cboe.com/api/global/us_indices/daily_prices/VIX_History.csv",
    )



import pandas as pd

from .data import data_path, date_to_decimal_year, asserted_dependency

__all__ = ["load"]


def load():
    _fetch()

    df = pd.read_csv(data_path("snp", "snp500_vol.csv"))
    df = df[["date", "vol"]]
    df.columns = ["date", "vol"]

    # Convert date to decimal years and make index.
    df["date"] = [date_to_decimal_year(date, "%Y-%m-%d") for date in df["date"]]
    df.set_index("date", inplace=True)

    return df


def _fetch():
    asserted_dependency(target=data_path("snp", "snp500_vol.csv"))
